@@ -43,12 +43,10 @@ namespace MonoDevelop.Database.ConnectionManager
 {
 	public class ConnectionContextNodeBuilder : TypeNodeBuilder
 	{
-		private EventHandler RefreshHandler;
-		
 		public ConnectionContextNodeBuilder ()
 			: base ()
 		{
-			RefreshHandler = new EventHandler (OnRefreshEvent);
+			
 		}
 		
 		public override Type NodeDataType {
@@ -74,12 +72,17 @@ namespace MonoDevelop.Database.ConnectionManager
 			return context.ConnectionSettings.Name;
 		}
 		
-		public override void BuildNode (ITreeBuilder builder, object dataObject, ref string label, ref Gdk.Pixbuf icon, ref Gdk.Pixbuf closedIcon)
+		public override void BuildNode (ITreeBuilder builder, object dataObject, ref string label, ref Gdk.Pixbuf icon, 
+		                                ref Gdk.Pixbuf closedIcon)
 		{
 			DatabaseConnectionContext context = dataObject as DatabaseConnectionContext;
-			context.RefreshEvent += (EventHandler)DispatchService.GuiDispatch (RefreshHandler);
+
+			if (((bool)builder.Options["ShowDatabaseName"]))
+				label = string.Format ("{0} - ({1})", 
+				                       context.ConnectionSettings.Name, context.ConnectionSettings.Database);
+			 else
+				label = context.ConnectionSettings.Name;
 			
-			label = context.ConnectionSettings.Name;
 			if (context.HasConnectionPool) {
 				IConnectionPool pool = context.ConnectionPool;
 				if (pool.IsInitialized) {
@@ -135,12 +138,6 @@ namespace MonoDevelop.Database.ConnectionManager
 		{
 			return true;
 		}
-		
-		private void OnRefreshEvent (object sender, EventArgs args)
-		{
-			ITreeBuilder builder = Context.GetTreeBuilder (sender);
-			builder.Update ();
-		}
 	}
 	
 	public class ConnectionContextCommandHandler : NodeCommandHandler
@@ -156,7 +153,7 @@ namespace MonoDevelop.Database.ConnectionManager
 					));
 				} else {
 					context.ConnectionSettings.Name = newName;
-					OnRefreshConnection ();
+					context.Refresh ();
 				}
 			}
 		}
@@ -176,20 +173,16 @@ namespace MonoDevelop.Database.ConnectionManager
 		protected void OnEditConnection ()
 		{
 			DatabaseConnectionContext context = (DatabaseConnectionContext) CurrentNode.DataItem;
-			DatabaseConnectionSettingsDialog dlg = new DatabaseConnectionSettingsDialog (context.DbFactory, context.ConnectionSettings);
-
-			if (dlg.Run () == (int)ResponseType.Ok) {
-				ConnectionContextService.EditDatabaseConnectionContext (context);
-				OnRefreshConnection ();
+			DatabaseConnectionSettings newSettings;
+			if (context.DbFactory.GuiProvider.ShowEditConnectionDialog (context.DbFactory,
+			                                                            context.ConnectionSettings,
+			                                                            out newSettings)) {
+				DatabaseConnectionContext newContext = new DatabaseConnectionContext (newSettings);
+				ConnectionContextService.RemoveDatabaseConnectionContext (context);
+				ConnectionContextService.AddDatabaseConnectionContext (newContext);
+				newContext.Refresh ();
 			}
-			dlg.Destroy ();
-		}
-		
-		[CommandHandler (ConnectionManagerCommands.Refresh)]
-		protected void OnRefreshConnection ()
-		{
-			DatabaseConnectionContext context = (DatabaseConnectionContext) CurrentNode.DataItem;
-			context.Refresh ();
+			
 		}
 		
 		[CommandHandler (ConnectionManagerCommands.DisconnectConnection)]
@@ -256,12 +249,18 @@ namespace MonoDevelop.Database.ConnectionManager
 		private void OnDropDatabaseThreaded (object state)
 		{
 			DatabaseConnectionContext context = (DatabaseConnectionContext) CurrentNode.DataItem;
-			ISchemaProvider provider = context.SchemaProvider;
-			DatabaseSchema db = provider.CreateDatabaseSchema (context.ConnectionSettings.Name);
-			
-			IEditSchemaProvider schemaProvider = (IEditSchemaProvider)context.SchemaProvider;
-			schemaProvider.DropDatabase (db);
-			DispatchService.GuiDispatch (delegate () { OnRefreshConnection (); });
+			try {
+				context.ConnectionPool.Initialize ();
+				ISchemaProvider provider = context.SchemaProvider;
+				DatabaseSchema db = provider.CreateDatabaseSchema (context.ConnectionSettings.Database);
+				IEditSchemaProvider schemaProvider = (IEditSchemaProvider)context.SchemaProvider;
+				schemaProvider.DropDatabase (db);
+				ConnectionContextService.RemoveDatabaseConnectionContext (context);
+			} catch (Exception ex) {
+				DispatchService.GuiDispatch (delegate {
+					MessageService.ShowException (ex);
+				});
+			}
 		}
 		
 		[CommandUpdateHandler (ConnectionManagerCommands.DropDatabase)]
@@ -290,5 +289,6 @@ namespace MonoDevelop.Database.ConnectionManager
 			DatabaseConnectionContext context = (DatabaseConnectionContext) CurrentNode.DataItem;
 			//info.Enabled = context.DbFactory.IsActionSupported ("Database", SchemaActions.Rename);
 		}
+	
 	}
 }
