@@ -40,12 +40,9 @@ namespace MonoDevelop.Database.ConnectionManager
 {
 	public class ParametersNodeBuilder : TypeNodeBuilder
 	{
-		private EventHandler RefreshHandler;
-		
 		public ParametersNodeBuilder ()
 			: base ()
 		{
-			RefreshHandler = new EventHandler (OnRefreshEvent);
 		}
 		
 		public override Type NodeDataType {
@@ -71,7 +68,6 @@ namespace MonoDevelop.Database.ConnectionManager
 			icon = Context.GetIcon ("md-db-tables");
 			
 			BaseNode node = (BaseNode) dataObject;
-			node.RefreshEvent += RefreshHandler;
 		}
 		
 		public override void BuildChildNodes (ITreeBuilder builder, object dataObject)
@@ -99,15 +95,6 @@ namespace MonoDevelop.Database.ConnectionManager
 			return true;
 		}
 		
-		private void OnRefreshEvent (object sender, EventArgs args)
-		{
-			ITreeBuilder builder = Context.GetTreeBuilder ();
-			
-			if (builder != null)
-				builder.UpdateChildren ();
-			
-			builder.ExpandToNode ();
-		}
 	}
 	
 	public class ParametersNodeCommandHandler : NodeCommandHandler
@@ -117,11 +104,5 @@ namespace MonoDevelop.Database.ConnectionManager
 			return DragOperation.None;
 		}
 		
-		[CommandHandler (ConnectionManagerCommands.Refresh)]
-		protected void OnRefresh ()
-		{
-			BaseNode node = CurrentNode.DataItem as BaseNode;
-			node.Refresh ();
-		}
 	}
 }
\ No newline at end of file
