@@ -40,12 +40,9 @@ namespace MonoDevelop.Database.ConnectionManager
 {
 	public class ProcedureNodeBuilder : TypeNodeBuilder
 	{
-		private EventHandler RefreshHandler;
-
 		public ProcedureNodeBuilder ()
 			: base ()
 		{
-			RefreshHandler = new EventHandler (OnRefreshEvent);
 		}
 		
 		public override Type NodeDataType {
@@ -74,10 +71,12 @@ namespace MonoDevelop.Database.ConnectionManager
 		public override void BuildNode (ITreeBuilder builder, object dataObject, ref string label, ref Gdk.Pixbuf icon, ref Gdk.Pixbuf closedIcon)
 		{
 			ProcedureNode node = dataObject as ProcedureNode;
-			node.RefreshEvent += (EventHandler)DispatchService.GuiDispatch (RefreshHandler);
-
+			
 			label = node.Procedure.Name;
-			icon = Context.GetIcon ("md-db-procedure");
+			if (node.Procedure.IsFunction)
+				icon = Context.GetIcon ("md-db-function");
+			else
+				icon = Context.GetIcon ("md-db-procedure");
 		}
 		
 		public override void BuildChildNodes (ITreeBuilder builder, object dataObject)
@@ -104,11 +103,6 @@ namespace MonoDevelop.Database.ConnectionManager
 			return node.ConnectionContext.SchemaProvider.IsSchemaActionSupported (SchemaType.ProcedureParameter, SchemaActions.Schema);
 		}
 		
-		private void OnRefreshEvent (object sender, EventArgs args)
-		{
-			ITreeBuilder builder = Context.GetTreeBuilder (sender);
-			builder.Update ();
-		}
 	}
 	
 	public class ProcedureNodeCommandHandler : NodeCommandHandler
@@ -147,13 +141,6 @@ namespace MonoDevelop.Database.ConnectionManager
 			node.Refresh ();
 		}
 		
-		[CommandHandler (ConnectionManagerCommands.Refresh)]
-		protected void OnRefresh ()
-		{
-			BaseNode node = CurrentNode.DataItem as BaseNode;
-			node.Refresh ();
-		}
-		
 		protected void OnRefreshParent ()
 		{
 			if (CurrentNode.MoveToParent ()) {
@@ -168,17 +155,19 @@ namespace MonoDevelop.Database.ConnectionManager
 			ProcedureNode node = CurrentNode.DataItem as ProcedureNode;
 			IDbFactory fac = node.ConnectionContext.DbFactory;
 			IEditSchemaProvider schemaProvider = (IEditSchemaProvider)node.ConnectionContext.SchemaProvider;
-			
-			if (fac.GuiProvider.ShowProcedureEditorDialog (schemaProvider, node.Procedure, false))
-				ThreadPool.QueueUserWorkItem (new WaitCallback (OnAlterProcedureThreaded), CurrentNode.DataItem);
+			ProcedureSchema alterProc = (ProcedureSchema)node.Procedure.Clone ();
+			if (fac.GuiProvider.ShowProcedureEditorDialog (schemaProvider, alterProc, false))
+				ThreadPool.QueueUserWorkItem (new WaitCallback (OnAlterProcedureThreaded), alterProc);
 		}
 		
 		private void OnAlterProcedureThreaded (object state)
 		{
-//			ProcedureNode node = (ProcedureNode)state;
-//			IEditSchemaProvider provider = (IEditSchemaProvider)node.ConnectionContext.SchemaProvider;
-//			
-//			provider.AlterProcedure (node.Procedure);
+			ProcedureNode node = CurrentNode.DataItem as ProcedureNode;
+			IDbFactory fac = node.ConnectionContext.DbFactory;
+			IEditSchemaProvider schemaProvider = (IEditSchemaProvider)node.ConnectionContext.SchemaProvider;
+			ProcedureAlterSchema alterSchema = new ProcedureAlterSchema (node.Procedure, (ProcedureSchema)((ProcedureSchema)state).Clone ());
+			schemaProvider.AlterProcedure (alterSchema);
+			OnRefreshParent ();
 		}
 		
 		[CommandHandler (ConnectionManagerCommands.DropProcedure)]
