@@ -43,11 +43,10 @@ namespace MonoDevelop.Database.ConnectionManager
 	public class ProceduresNodeBuilder : TypeNodeBuilder
 	{
 		private EventHandler RefreshHandler;
-		
 		public ProceduresNodeBuilder ()
 			: base ()
 		{
-			RefreshHandler = new EventHandler (OnRefreshEvent);
+			RefreshHandler += new EventHandler(OnRefreshEvent);
 		}
 		
 		public override Type NodeDataType {
@@ -73,7 +72,7 @@ namespace MonoDevelop.Database.ConnectionManager
 			icon = Context.GetIcon ("md-db-procedure");
 			
 			BaseNode node = (BaseNode) dataObject;
-			node.RefreshEvent += (EventHandler)(DispatchService.GuiDispatch (RefreshHandler));
+			node.RefreshEvent += RefreshHandler;
 		}
 		
 		public override void BuildChildNodes (ITreeBuilder builder, object dataObject)
@@ -104,13 +103,13 @@ namespace MonoDevelop.Database.ConnectionManager
 			return true;
 		}
 		
-		private void OnRefreshEvent (object sender, EventArgs args)
-		{
-			ITreeBuilder builder = Context.GetTreeBuilder ();
-			
-			builder.UpdateChildren ();			
-			builder.ExpandToNode ();
-		}
+ 		private void OnRefreshEvent (object sender, EventArgs args)
+	 	{
+			DispatchService.GuiDispatch (delegate {
+	 			ITreeBuilder builder = Context.GetTreeBuilder (sender);
+				builder.UpdateChildren ();
+			});
+	 	}
 	}
 	
 	public class ProceduresNodeCommandHandler : NodeCommandHandler
@@ -120,13 +119,6 @@ namespace MonoDevelop.Database.ConnectionManager
 			return DragOperation.None;
 		}
 		
-		[CommandHandler (ConnectionManagerCommands.Refresh)]
-		protected void OnRefresh ()
-		{
-			BaseNode node = CurrentNode.DataItem as BaseNode;
-			node.Refresh ();
-		}
-		
 		[CommandHandler (ConnectionManagerCommands.CreateProcedure)]
 		protected void OnCreateProcedure ()
 		{
