@@ -43,11 +43,10 @@ namespace MonoDevelop.Database.ConnectionManager
 	public class UsersNodeBuilder : TypeNodeBuilder
 	{
 		private EventHandler RefreshHandler;
-		
 		public UsersNodeBuilder ()
 			: base ()
 		{
-			RefreshHandler = new EventHandler (OnRefreshEvent);
+			RefreshHandler += new EventHandler(OnRefreshEvent);
 		}
 		
 		public override Type NodeDataType {
@@ -73,7 +72,7 @@ namespace MonoDevelop.Database.ConnectionManager
 			icon = Context.GetIcon ("md-db-users");
 			
 			BaseNode node = (BaseNode) dataObject;
-			node.RefreshEvent += (EventHandler)(DispatchService.GuiDispatch (RefreshHandler));
+			RefreshHandler += new EventHandler(OnRefreshEvent);
 		}
 		
 		public override void BuildChildNodes (ITreeBuilder builder, object dataObject)
@@ -98,14 +97,14 @@ namespace MonoDevelop.Database.ConnectionManager
 		{
 			return true;
 		}
-		
-		private void OnRefreshEvent (object sender, EventArgs args)
-		{
-			ITreeBuilder builder = Context.GetTreeBuilder (sender);
-			
-			builder.UpdateChildren ();			
-			builder.ExpandToNode ();
-		}
+
+ 		private void OnRefreshEvent (object sender, EventArgs args)
+	 	{
+			DispatchService.GuiDispatch (delegate {
+	 			ITreeBuilder builder = Context.GetTreeBuilder (sender);
+				builder.UpdateChildren ();
+			});
+	 	}
 	}
 	
 	public class UsersNodeCommandHandler : NodeCommandHandler
@@ -115,13 +114,6 @@ namespace MonoDevelop.Database.ConnectionManager
 			return DragOperation.None;
 		}
 		
-		[CommandHandler (ConnectionManagerCommands.Refresh)]
-		protected void OnRefresh ()
-		{
-			BaseNode node = CurrentNode.DataItem as BaseNode;
-			node.Refresh ();
-		}
-		
 		[CommandHandler (ConnectionManagerCommands.CreateUser)]
 		protected void OnCreateUser ()
 		{
