@@ -1,4 +1,4 @@
-﻿//
+//
 // Authors:
 //   Ben Motmans  <ben.motmans@gmail.com>
 //
@@ -25,6 +25,7 @@
 
 using Gtk;
 using System;
+using System.IO;
 using System.Threading;
 using System.Collections.Generic;
 using MonoDevelop.Core;
@@ -46,16 +47,15 @@ namespace MonoDevelop.Database.Designer
 		
 		private CommentEditorWidget commentEditor;
 		private SqlEditorWidget sqlEditor;
+		private ProcedureEditorSettings settings;
 		
 		public ProcedureEditorDialog (IEditSchemaProvider schemaProvider, bool create, ProcedureEditorSettings settings)
 		{
 			if (schemaProvider == null)
 				throw new ArgumentNullException ("schemaProvider");
-			if (procedure == null)
-				throw new ArgumentNullException ("procedure");
 			
+			this.settings = settings;
 			this.schemaProvider = schemaProvider;
-			this.procedure = procedure;
 			this.action = create ? SchemaActions.Create : SchemaActions.Alter;
 			
 			this.Build();
@@ -75,6 +75,11 @@ namespace MonoDevelop.Database.Designer
 				commentEditor = new CommentEditorWidget ();
 				notebook.AppendPage (commentEditor, new Label (AddinCatalog.GetString ("Comment")));
 			}
+			
+			if (!settings.ShowName) {
+				nameLabel.Visible = false;
+				entryName.Visible = false;
+			}
 
 			vboxContent.PackStart (notebook, true, true, 0);
 			vboxContent.ShowAll ();
@@ -83,20 +88,28 @@ namespace MonoDevelop.Database.Designer
 		
 		public void Initialize (ProcedureSchema procedure)
 		{
+			if (procedure == null)
+				throw new ArgumentNullException ("procedure");
+			
+			this.procedure = procedure;
 			entryName.Text = procedure.Name;
 
-			if (action == SchemaActions.Create) {
+			if (action == SchemaActions.Alter) {
 				sqlEditor.Text = schemaProvider.GetProcedureAlterStatement (procedure);
 				if (commentEditor != null)
 					commentEditor.Comment = procedure.Comment;
-			}
+			} else 
+				sqlEditor.Text = procedure.Definition;
 		}
 
 		protected virtual void OkClicked (object sender, EventArgs e)
 		{
-			procedure.Name = entryName.Text;
+			if (settings.ShowName)
+				procedure.Name = entryName.Text;
 			procedure.Definition = sqlEditor.Text;
-			procedure.Comment = commentEditor.Comment;
+			
+			if (settings.ShowComment)
+				procedure.Comment = commentEditor.Comment;
 			
 			Respond (ResponseType.Ok);
 			Hide ();
@@ -134,15 +147,96 @@ namespace MonoDevelop.Database.Designer
 				labelWarning.Text = msg;
 			}
 		}
+		protected virtual void OnButtonSaveClicked (object sender, System.EventArgs e)
+		{
+			FileChooserDialog dlg = new FileChooserDialog (
+				AddinCatalog.GetString ("Save Script"), null, FileChooserAction.Save,
+				"gtk-cancel", ResponseType.Cancel,
+				"gtk-save", ResponseType.Accept
+			);
+			dlg.SelectMultiple = false;
+			dlg.LocalOnly = true;
+			dlg.Modal = true;
+		
+			FileFilter filter = new FileFilter ();
+			filter.AddPattern ("*.sql");
+			filter.Name = AddinCatalog.GetString ("SQL Scripts");
+			FileFilter filterAll = new FileFilter ();
+			filterAll.AddPattern ("*");
+			filterAll.Name = AddinCatalog.GetString ("All files");
+			dlg.AddFilter (filter);
+			dlg.AddFilter (filterAll);
+
+			try {
+				if (dlg.Run () == (int)ResponseType.Accept) {
+					if (File.Exists (dlg.Filename)) {
+						if (!MessageService.Confirm (AddinCatalog.GetString (@"File {0} already exists. 
+													Do you want to overwrite\nthe existing file?", dlg.Filename), 
+						                             AlertButton.Yes))
+							return;
+						else
+							File.Delete (dlg.Filename);
+					}
+				 	using (StreamWriter writer =  File.CreateText (dlg.Filename)) {
+						writer.Write (sqlEditor.Text);
+						writer.Close ();
+					}
+					
+				}
+			} finally {
+				dlg.Destroy ();					
+			}
+		}
+		
+		protected virtual void OnButtonOpenClicked (object sender, System.EventArgs e)
+		{
+			FileChooserDialog dlg = new FileChooserDialog (
+				AddinCatalog.GetString ("Open Script"), null, FileChooserAction.Open,
+				"gtk-cancel", ResponseType.Cancel,
+				"gtk-open", ResponseType.Accept
+			);
+			dlg.SelectMultiple = false;
+			dlg.LocalOnly = true;
+			dlg.Modal = true;
+		
+			FileFilter filter = new FileFilter ();
+			filter.AddPattern ("*.sql");
+			filter.Name = AddinCatalog.GetString ("SQL Scripts");
+			FileFilter filterAll = new FileFilter ();
+			filterAll.AddPattern ("*");
+			filterAll.Name = AddinCatalog.GetString ("All files");
+			dlg.AddFilter (filter);
+			dlg.AddFilter (filterAll);
+
+			try {
+				if (dlg.Run () == (int)ResponseType.Accept) {
+					
+				 	using (StreamReader reader =  File.OpenText (dlg.Filename)) {
+						sqlEditor.Text = reader.ReadToEnd ();
+						reader.Close ();
+					}
+					
+				}
+			} finally {
+				dlg.Destroy ();					
+			}
+		}
+		
 	}
 	
 	public class ProcedureEditorSettings
 	{
-		private bool showComment = false;
-
+		bool showComment = false;
+		bool showName = true;
+		
 		public bool ShowComment {
 			get { return showComment; }
 			set { showComment = value; }
 		}
+		
+		public bool ShowName {
+			get { return showName; }
+			set { showName = value; }
+		}
 	}
 }
