@@ -29,6 +29,7 @@
 
 using Gtk;
 using System;
+using System.IO;
 using System.Data;
 using System.Threading;
 using System.Collections.Generic;
@@ -38,14 +39,15 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Components;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Components.Commands;
 using MonoDevelop.Database.Components;
+using MonoDevelop.SourceEditor;
 
 namespace MonoDevelop.Database.Query
 {
-	public class SqlQueryView : AbstractViewContent
+	public class SqlQueryView : SourceEditorView, ISqlQueryEditorView
 	{
 		private VBox vbox;
-		private SqlEditorWidget sqlEditor;
 		private ToolButton buttonExecute;
 		private ToolButton buttonStop;
 		private ToolButton buttonClear;
@@ -53,6 +55,7 @@ namespace MonoDevelop.Database.Query
 		private Notebook notebook;
 		private VPaned pane;
 		private TextView status;
+		SqlEditorWidget history;
 		
 		private object currentQueryState;
 		private List<object> stoppedQueries;
@@ -63,35 +66,36 @@ namespace MonoDevelop.Database.Query
 		public SqlQueryView ()
 		{
 			stoppedQueries = new List<object> ();
+			MonoDevelop.SourceEditor.Extension.TemplateExtensionNodeLoader.Init ();
+			this.UntitledName = string.Concat (AddinCatalog.GetString ("Untitled Sql Script"), ".sql");
 			
 			vbox = new VBox (false, 6);
 			vbox.BorderWidth = 6;
-
-			sqlEditor = new SqlEditorWidget ();
-			sqlEditor.TextChanged += new EventHandler (SqlChanged);
 			
 			Toolbar toolbar = new Toolbar ();
 			toolbar.ToolbarStyle = ToolbarStyle.BothHoriz;
 			
-			buttonExecute = new ToolButton (
-				Services.Resources.GetImage ("md-db-execute", IconSize.SmallToolbar),
-					AddinCatalog.GetString ("Execute")
-			);
-			buttonStop = new ToolButton ("gtk-stop");
-			buttonClear = new ToolButton (Services.Resources.GetImage ("gtk-clear", IconSize.Button), AddinCatalog.GetString ("Clear Results"));
-			buttonStop.Sensitive = false;
+			buttonExecute = new ToolButton (ImageService.GetImage ("md-db-execute", IconSize.SmallToolbar),
+			                                AddinCatalog.GetString ("_Execute"));
+			buttonExecute.Label = AddinCatalog.GetString ("Execute");
 			buttonExecute.Sensitive = false;
-			
+			buttonExecute.TooltipMarkup = AddinCatalog.GetString ("Execute Query");
+			buttonExecute.IsImportant = true;
 			buttonExecute.Clicked += new EventHandler (ExecuteClicked);
+
+			buttonStop = new ToolButton ("gtk-stop");
+			buttonStop.TooltipText = AddinCatalog.GetString ("Stop Query Execution");
+			buttonStop.Sensitive = false;
 			buttonStop.Clicked += new EventHandler (StopClicked);
+			
+			buttonClear = new ToolButton (ImageService.GetImage ("gtk-clear", IconSize.Button), 
+			                              AddinCatalog.GetString ("Clear Results"));
+			buttonClear.TooltipText = AddinCatalog.GetString ("Clear Results");
 			buttonClear.Clicked += new EventHandler (ClearClicked);
 			
 			comboConnections = new DatabaseConnectionContextComboBox ();
 			selectedConnection = comboConnections.DatabaseConnection;
 			comboConnections.Changed += new EventHandler (ConnectionChanged);
-
-			buttonExecute.IsImportant = true;
-			
 			ToolItem comboItem = new ToolItem ();
 			comboItem.Child = comboConnections;
 			
@@ -103,26 +107,76 @@ namespace MonoDevelop.Database.Query
 			
 			pane = new VPaned ();
 
+			// Sql History Window
+			ScrolledWindow windowHistory = new ScrolledWindow ();
+			history = new SqlEditorWidget ();
+			history.Editable = false;
+			windowHistory.AddWithViewport (history);
+			
+			// Status of the Last Query
 			ScrolledWindow windowStatus = new ScrolledWindow ();
 			status = new TextView ();
 			windowStatus.Add (status);
 			
 			notebook = new Notebook ();
 			notebook.AppendPage (windowStatus, new Label (AddinCatalog.GetString ("Status")));
+			notebook.AppendPage (windowHistory, new Label (AddinCatalog.GetString ("Query History")));
 			
-			pane.Pack1 (sqlEditor, true, true);
 			pane.Pack2 (notebook, true, true);
-			
 			vbox.PackStart (toolbar, false, true, 0);
 			vbox.PackStart (pane, true, true, 0);
-			
+			this.Document.TextReplaced += SqlChanged;
 			vbox.ShowAll ();
+			this.Document.DocumentUpdated += delegate (object sender, EventArgs args) {
+				// Set the MimeType when the file is saved.
+				this.Document.MimeType = "text/x-sql";
+			};
 			notebook.Hide ();
+		} 
+
+		#region ISqlQueryEditorView implementation
+		[CommandHandler (MonoDevelop.Debugger.DebugCommands.Debug)]
+		public void RunQuery ()
+		{
+			ExecuteClicked (new object (), new EventArgs ());
 		}
 		
-		public string Text {
-			get { return sqlEditor.Text; }
-			set { sqlEditor.Text = value; }
+		[CommandUpdateHandler (MonoDevelop.Debugger.DebugCommands.Debug)]		
+		protected void UpdateQueryCommand (CommandInfo info)
+		{
+			info.Icon = "md-db-execute";
+			info.Text ="Execute _Query";
+		}
+		
+		public void StopQuery ()
+		{
+			StopClicked (new object (), new EventArgs ());
+		}
+		
+		public bool IsRunning {
+			get {
+				if (currentQueryState == null)
+					return false;
+				else
+					return true;
+			}
+		}
+		
+		public void ClearQuery ()
+		{
+			ClearClicked (new object (), new EventArgs ());
+		}
+		#endregion
+				
+		public string QueryText {
+			get { 
+				
+				if (this.SelectedText != string.Empty)
+					return this.SelectedText;
+				else
+					return this.Text;
+			}
+			set { this.Text = value; }
 		}
 		
 		public DatabaseConnectionContext SelectedConnectionContext {
@@ -135,28 +189,32 @@ namespace MonoDevelop.Database.Query
 				}
 			}
 		}
-		
-		public override string UntitledName {
-			get { return "SQL Query"; }
-		}
-		
+
 		public override void Dispose ()
 		{
 			Control.Destroy ();
 		}
 		
-		public override void Load (string filename)
+		internal void CreateQueryWidget (Widget control)
 		{
-			throw new NotImplementedException ();
+			if (!(control is VBox))
+				return;
+			
+			((VBox)control).PackEnd (vbox, true, true, 0);
+			
 		}
 		
 		public override Widget Control {
-			get { return vbox; }
+			get { 
+				if (vbox.Parent == null)
+					CreateQueryWidget (base.Control);
+				return base.Control;
+			}
 		}
 		
 		public void ExecuteQuery ()
 		{
-			if (selectedConnection == null || Text.Length < 0) {
+			if (selectedConnection == null || QueryText.Length < 0) {
 				SetQueryState (false, "No connection selected.");
 				return;
 			}
@@ -174,7 +232,11 @@ namespace MonoDevelop.Database.Query
 			
 			currentQueryState = new object ();
 			IPooledDbConnection conn = context.ConnectionPool.Request ();
-			IDbCommand command = conn.CreateCommand (Text);
+			IDbCommand command = conn.CreateCommand (QueryText);
+			if (history.Text.EndsWith (Environment.NewLine) || history.Text == string.Empty)
+				history.Text = string.Concat (history.Text, QueryText);
+			else
+				history.Text = string.Concat (history.Text, Environment.NewLine, "------------------------", QueryText);
 			queryStart = DateTime.Now;
 			conn.ExecuteSetAsync (command, new ExecuteCallback<DataSet> (ExecuteQueryThreaded), currentQueryState);
 		}
@@ -183,7 +245,7 @@ namespace MonoDevelop.Database.Query
 		{
 			connection.Release ();
 			TimeSpan duration = DateTime.Now.Subtract (queryStart);
-
+			
 			DispatchService.GuiDispatch (delegate () {
 				notebook.ShowAll ();
 				string msg = String.Concat (
@@ -210,19 +272,24 @@ namespace MonoDevelop.Database.Query
 						string msg = String.Concat (Environment.NewLine, AddinCatalog.GetString ("Table"), ": ",table.TableName,
 							Environment.NewLine, "\t", AddinCatalog.GetString ("Affected Rows"), ": ", table.Rows.Count);
 						status.Buffer.Text += msg;
-							
-						TabLabel label = new TabLabel (new Label (table.TableName), Services.Resources.GetImage ("md-db-table", IconSize.Menu));
+						
+						TabLabel label = new TabLabel (new Label (table.TableName), ImageService.GetImage ("md-db-table", IconSize.Menu));
 						label.CloseClicked += new EventHandler (OnResultTabClose);
 						notebook.AppendPage (grid, label);
 						notebook.ShowAll ();
+						this.Document.ReadOnly = false;
+						notebook.Page = notebook.NPages -1;
 					});
 				}
+				
 			}
 			
 			if (result == null || result.Tables.Count == 0) {
 				DispatchService.GuiDispatch (delegate () {
 					status.Buffer.Text += AddinCatalog.GetString ("No Results");
+					this.Document.ReadOnly = false;
 				});
+				
 			}
 		}
 			
@@ -245,10 +312,9 @@ namespace MonoDevelop.Database.Query
 		
 		private void ClearClicked (object sender, EventArgs e)
 		{
-			for (int i=1; i<notebook.NPages; i++)
-				notebook.RemovePage (i);
+			while (notebook.NPages > 2)
+				notebook.RemovePage (2);
 			status.Buffer.Text = String.Empty;
-			notebook.Hide ();
 		}
 		
 		private void StopClicked (object sender, EventArgs e)
@@ -265,24 +331,25 @@ namespace MonoDevelop.Database.Query
 		private void ConnectionChanged (object sender, EventArgs args)
 		{
 			selectedConnection = comboConnections.DatabaseConnection;
-			buttonExecute.Sensitive = sqlEditor.Text.Length > 0;
+			buttonExecute.Sensitive = QueryText.Length > 0;
 		}
 		
-		private void SqlChanged (object sender, EventArgs args)
+		private void SqlChanged (object sender, Mono.TextEditor.ReplaceEventArgs args)
 		{
-			buttonExecute.Sensitive = sqlEditor.Text.Length > 0;
+			buttonExecute.Sensitive = QueryText.Length > 0;
 		}
 		
 		private void SetQueryState (bool exec, string msg)
 		{
-			buttonExecute.Sensitive = !exec && sqlEditor.Text.Length > 0;
+			buttonExecute.Sensitive = !exec && QueryText.Length > 0;
 			buttonStop.Sensitive = exec;
 			buttonClear.Sensitive = !exec;
-			sqlEditor.Editable = !exec;
+			this.Document.ReadOnly = !exec;
 			notebook.Show ();
-			
 			status.Buffer.Text = msg + Environment.NewLine;
 		}
+
+
 	}
 	
 	internal class ConnectionContextMenuItem : RadioMenuItem
@@ -304,4 +371,4 @@ namespace MonoDevelop.Database.Query
 			(Child as Label).Text = context.ConnectionSettings.Name;
 		}
 	}
-}
\ No newline at end of file
+}
