@@ -59,8 +59,9 @@ namespace MonoDevelop.Database.Sql.Sqlite
 			TableSchemaCollection tables = new TableSchemaCollection ();
 			
 			IPooledDbConnection conn = connectionPool.Request ();
+			// Tables need to be ordered bacause TableSchemaCollection is "created" as sorted by default.
 			IDbCommand command = conn.CreateCommand (
-				"SELECT name, sql FROM sqlite_master WHERE type = 'table'"
+				"SELECT name, sql FROM sqlite_master WHERE type = 'table' ORDER BY name"
 			);
 			try {
 				using (command) {
@@ -160,6 +161,76 @@ namespace MonoDevelop.Database.Sql.Sqlite
 			return GetConstraints (table, column);
 		}
 
+		public override TriggerSchemaCollection GetTableTriggers (TableSchema table)
+		{
+			
+			if (table == null)
+				throw new ArgumentNullException ("table");
+			TriggerSchemaCollection triggers = new TriggerSchemaCollection ();
+			
+			IPooledDbConnection conn = connectionPool.Request ();
+
+			IDbCommand command = conn.CreateCommand (string.Format (@"SELECT * FROM sqlite_master
+																	WHERE type = 'trigger' and tbl_name = '{0}'", 
+			                                                        table.Name));
+			try {
+				// Parse Trigger Sql - Needed for alter a table
+				System.Text.RegularExpressions.Regex parseRegEx = new System.Text.RegularExpressions.Regex 
+														(string.Concat (
+					                					@"((CREATE\s*(Temp|Temporary)?\s*TRIGGER){1}\s?(\w+)\s?(IF NOT",
+														@" EXISTS)?\s?(BEFORE|AFTER|INSTEAD OF){1}\s?(\w+)\s*ON(\s+\w*",
+														@")\s*(FOR EACH ROW){1}\s*(BEGIN){1})\s+(\w|\W)*(END)"));
+				using (IDataReader r = command.ExecuteReader ()) {
+					while (r.Read ()) {
+						string sql = r.GetString (r.GetOrdinal ("sql"));
+						System.Text.RegularExpressions.MatchCollection matchs = parseRegEx.Matches (sql);
+						TriggerSchema trigger = new TriggerSchema (this);
+						trigger.Name = r.GetString (r.GetOrdinal ("name"));
+						trigger.TableName = r.GetString (r.GetOrdinal ("tbl_name"));
+						trigger.Definition = sql;
+						if (matchs.Count > 0) {
+							trigger.TriggerFireType = TriggerFireType.ForEachRow;
+							switch (matchs[0].Groups[7].Value.ToLower ()) {
+								case "insert":
+									trigger.TriggerEvent = TriggerEvent.Insert;
+									break;
+								case "update":
+									trigger.TriggerEvent = TriggerEvent.Update;
+									break;
+								case "delete":
+									trigger.TriggerEvent = TriggerEvent.Delete;
+									break;
+								default:
+									throw new NotImplementedException ();
+							}
+							
+							switch (matchs[0].Groups[7].Value.ToLower ()) {
+								case "before":
+									trigger.TriggerType = TriggerType.Before;
+									break;
+								case "after":
+									trigger.TriggerType = TriggerType.After;
+									break;
+								default:
+									throw new NotImplementedException ();
+							}
+							StringBuilder sbSource = new StringBuilder ();
+							foreach (System.Text.RegularExpressions.Capture c in matchs[0].Groups[11].Captures)
+								sbSource.Append (c.Value);
+							trigger.Source = sbSource.ToString ();
+							
+							
+						}
+						triggers.Add (trigger);
+					}
+				}
+			} catch (SqliteException e) {
+				QueryService.RaiseException (e);
+			}
+			return triggers;
+		}
+
+		
 		//http://www.sqlite.org/pragma.html
 		public virtual ConstraintSchemaCollection GetConstraints (TableSchema table, ColumnSchema column)
 		{
@@ -231,15 +302,14 @@ namespace MonoDevelop.Database.Sql.Sqlite
 		
 		public override void CreateDatabase (DatabaseSchema database)
 		{
-			if (System.IO.File.Exists (database.Name)) {
-				MonoDevelop.Core.Gui.MessageService.ShowError (
-					AddinCatalog.GetString ("Database '{0}' already exists.", database.Name)
-				);
-				return;
+			if (System.IO.File.Exists (database.Name))
+				throw new SqliteDatabaseAlreadyExistsException (AddinCatalog.GetString 
+				                                                ("Database '{0}' already exists.", database.Name));
+			using (SqliteConnection conn = new SqliteConnection (string.Format ("URI=file:{0};Version=3;",
+			                                                                    database.Name))) {
+				conn.Open ();
+				conn.Close ();
 			}
-			SqliteConnection conn = new SqliteConnection ("URI=file:" + database.Name + ";Version=3;");
-			conn.Open ();
-			conn.Close ();
 		}
 
 		//http://www.sqlite.org/lang_createtable.html
@@ -273,9 +343,11 @@ namespace MonoDevelop.Database.Sql.Sqlite
 					sb.Append (GetConstraintString (constraint));
 				}
 			}
-			
 			//table constraints
 			foreach (ConstraintSchema constraint in table.Constraints) {
+				// SQLite create a new schema with an auto-index by default.
+				if (constraint.Name.StartsWith ("sqlite_autoindex"))
+					continue;
 				sb.Append (", ");
 				sb.Append (GetConstraintString (constraint));
 			}
@@ -292,24 +364,35 @@ namespace MonoDevelop.Database.Sql.Sqlite
 		
 		protected virtual string GetConstraintString (ConstraintSchema constraint)
 		{
+			bool first = true;
 			//PRIMARY KEY [sort-order] [ conflict-clause ] [AUTOINCREMENT]
 			//UNIQUE [ conflict-clause ]
 			//CHECK ( expr )
 			//COLLATE collation-name
 			
 			StringBuilder sb = new StringBuilder ();
-			sb.Append ("CONSTRAINT ");
-			sb.Append (constraint.Name);
-			sb.Append (' ');
-			
 			switch (constraint.ConstraintType) {
 			case ConstraintType.PrimaryKey:
-				//PrimaryKeyConstraintSchema pk = constraint as PrimaryKeyConstraintSchema;
-				sb.Append ("PRIMARY KEY"); //TODO: auto inc + sort
+				sb.Append ("PRIMARY KEY ("); //TODO: auto inc + sort
+				first = true;
+				foreach (ColumnSchema col in constraint.Columns) {
+					if (!first)
+						sb.Append (",");
+					sb.Append (col.Name);
+					first = false;
+				}
+				sb.Append (")");
 				break;
 			case ConstraintType.Unique:
-				//UniqueConstraintSchema u = constraint as UniqueConstraintSchema;
-				sb.Append ("UNIQUE");
+				sb.Append ("UNIQUE (");
+				first = true;
+				foreach (ColumnSchema col in constraint.Columns) {
+					if (!first)
+						sb.Append (",");
+					sb.Append (col.Name);
+					first = false;
+				}
+				sb.Append (")");
 				break;
 			case ConstraintType.Check:
 				CheckConstraintSchema chk = constraint as CheckConstraintSchema;
@@ -317,6 +400,28 @@ namespace MonoDevelop.Database.Sql.Sqlite
 				sb.Append (chk.Source);
 				sb.Append (")");
 				break;
+			case ConstraintType.ForeignKey:
+				sb.Append ("FOREIGN KEY ");
+				sb.Append (GetColumnsString (constraint.Columns, true));
+				sb.Append (" REFERENCES ");
+				
+				ForeignKeyConstraintSchema fk = constraint as ForeignKeyConstraintSchema;
+				string tableName;
+				if (fk.ReferenceTableName.IndexOf ('.') > 0)
+					 tableName = fk.ReferenceTableName.Substring (fk.ReferenceTableName.IndexOf ('.') + 1);
+				else
+					tableName = fk.ReferenceTableName;
+				sb.Append (tableName);
+				sb.Append (' ');
+				if (fk.ReferenceColumns != null)
+					sb.Append (GetColumnsString (fk.ReferenceColumns, true));
+				sb.Append (Environment.NewLine);
+				sb.Append (" ON DELETE ");
+				sb.Append (GetConstraintActionString (fk.DeleteAction));
+				sb.Append (Environment.NewLine);
+				sb.Append (" ON UPDATE ");
+				sb.Append (GetConstraintActionString (fk.UpdateAction));
+				break;
 			default:
 				throw new NotImplementedException ();
 			}
@@ -350,7 +455,6 @@ namespace MonoDevelop.Database.Sql.Sqlite
 		protected virtual string GetTriggerCreateStatement (TriggerSchema trigger)
 		{
 			StringBuilder sb = new StringBuilder ();
-			
 			sb.Append ("CREATE TRIGGER ");
 			sb.Append (trigger.Name);
 			
