@@ -76,6 +76,15 @@ namespace MonoDevelop.Database.Sql
 			this.supportedActions = new Dictionary<SchemaType, SchemaActions> ();
 		}
 		
+		public virtual string GetComentSeparator ()
+		{
+			return "--";
+		}
+		
+		public virtual bool CanComment {
+			get { return true; }
+		}
+			
 		public virtual bool CanEdit
 		{
 			get { return true; }
@@ -114,6 +123,26 @@ namespace MonoDevelop.Database.Sql
 			return schema;
 		}
 
+		public virtual string GetConstraintActionString (ForeignKeyAction action)
+		{
+			switch (action) {
+			case ForeignKeyAction.None:
+				return " NO ACTION ";
+			case ForeignKeyAction.Cascade:
+				return " CASCADE ";
+			case ForeignKeyAction.Restrict:
+				return " RESTRICT ";
+			case ForeignKeyAction.NoAction:
+				return " NO ACTION ";
+			case ForeignKeyAction.SetDefault:
+				return " SET DEFAULT ";
+			case ForeignKeyAction.SetNull:
+				return " SET NULL ";
+			default:
+				throw new NotImplementedException ();
+			}
+		}
+		
 		public virtual TableSchemaCollection GetTables ()
 		{
 			TableSchemaCollection collection = new TableSchemaCollection ();
@@ -127,7 +156,7 @@ namespace MonoDevelop.Database.Sql
 					collection.Add (GetTable (row));
 				}
 			} catch (Exception e) {
-				QueryService.RaiseException (e);
+				// Don't raise error, if the triggers doesn't exists return an empty collection
 			}
 			conn.Release ();
 			
@@ -769,10 +798,12 @@ namespace MonoDevelop.Database.Sql
 		
 		protected virtual int ExecuteNonQuery (string sql)
 		{
-			IPooledDbConnection conn = connectionPool.Request ();
-			int result = conn.ExecuteNonQuery (sql);
-			conn.Release ();
-			return result;
+			using (IPooledDbConnection conn = connectionPool.Request ())
+			{
+				int result = conn.ExecuteNonQuery (sql);
+				conn.Release ();
+				return result;
+			}
 		}
 	}
 }
